Package: makecdfenv
Version: 1.88.0
Date: 2006-03-06
Title: CDF Environment Maker
Author: Rafael A. Irizarry <rafa@jhu.edu>, Laurent Gautier
        <laurent@cbs.dtu.dk>, Wolfgang Huber
        <w.huber@dkfz-heidelberg.de>, Ben Bolstad <bmb@bmbolstad.com>
Maintainer: James W. MacDonald <jmacdon@u.washington.edu>
Depends: R (>= 2.6.0), affyio
Imports: Biobase, affy, methods, stats, utils
Description: This package has two functions. One reads a Affymetrix
        chip description file (CDF) and creates a hash table
        environment containing the location/probe set membership
        mapping. The other creates a package that automatically loads
        that environment.
License: GPL (>= 2)
LazyLoad: yes
biocViews: OneChannel, DataImport, Preprocessing
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:30:06 UTC
RemoteUrl: https://github.com/bioc/makecdfenv
RemoteRef: RELEASE_3_23
RemoteSha: 1ae1a3cf4b27c9035ae57f9d435c1692e6415836
NeedsCompilation: yes
Packaged: 2026-04-29 23:10:39 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 23:14:48 UTC; unix

Package: makecdfenv
Version: 1.88.0
Date: 2006-03-06
Title: CDF Environment Maker
Author: Rafael A. Irizarry <rafa@jhu.edu>, Laurent Gautier
        <laurent@cbs.dtu.dk>, Wolfgang Huber
        <w.huber@dkfz-heidelberg.de>, Ben Bolstad
        <bmb@bmbolstad.com>
Maintainer: James W. MacDonald <jmacdon@u.washington.edu>
Depends: R (>= 2.6.0), affyio
Imports: Biobase, affy, methods, stats, utils
Description: This package has two functions. One reads a Affymetrix
        chip description file (CDF) and creates a hash table
        environment containing the location/probe set membership
        mapping. The other creates a package that automatically loads
        that environment.
License: GPL (>= 2)
LazyLoad: yes
biocViews: OneChannel, DataImport, Preprocessing
git_url: https://git.bioconductor.org/packages/makecdfenv
git_branch: RELEASE_3_23
git_last_commit: 1ae1a3c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 22:59:37 UTC; biocbuild

Package: @PKGNAME@
Title: @PKGNAME@
Version: @VERSION@
Created: @TODAY@
Author: @AUTHOR@
Description: A package containing an environment representing the @CDFFILENAME@ file.
Maintainer: @MAINTAINER@
License: LGPL
biocViews: AnnotationData, AffymetrixChip, cdf, @PKGROOT@, @SPECIES@
Depends: utils
Imports: AnnotationDbi

useDynLib(makecdfenv)

import(methods)

importFrom(affy, cleancdfname)

importFrom(Biobase, createPackage, multiassign)

importFrom(stats, runif)

importFrom(utils, packageDescription)

exportClasses(Cdf)

exportMethods("name.levels<-", name.levels, "pbase.levels<-", pbase.levels, "pbase<-", pbase, show, "tbase.levels<-", tbase.levels, "tbase<-", tbase)

export(getInfoInFile, make.cdf.env, make.cdf.package, pmormm, read.cdffile)

import(affyio)

getInfoInFile <- function(filename, type, unit, property, compress=NULL) {
  if (is.null(compress) && (type == "CDF"))
    compress <- getOption("BioC")$affy$compress.cdf
  if (is.null(compress) && (type == "CEL"))
    compress <- getOption("BioC")$affy$compress.cel
  .Call("getInfo", as.character(filename), as.character(type),
  as.character(unit), as.character(property), as.integer(compress),
  PACKAGE="makecdfenv")
}

##------------------------------------------------------------
## (C) Rafael Irizarry, Wolfgang Huber 2003
##
## Modifications by B. M. Bolstad for binary CDF files Feb 2005
## Note that the binary parser ignores the cdffile object
## altogether and creates much of the structure in the c code
##
##------------------------------------------------------------
make.cdf.env <- function(filename,
                         cdf.path = getwd(),
                         compress = FALSE,
                         return.env.only = TRUE,
                         verbose = TRUE) {
  stopifnot(is.logical(verbose), is.logical(return.env.only), is.logical(compress),
            is.character(cdf.path), is.character(filename))
  stopifnot(length(filename)==1, length(cdf.path)==1, length(return.env.only)==1,
            length(verbose)==1, length(compress)==1)


  isCDFXDA <- function(filename){
    return (.Call("CheckCDFXDA",filename,PACKAGE="affyio"))
  }



  if (!isCDFXDA(file.path(path.expand(cdf.path),filename))){
    # a "classic" CDF file (ie text)

    ## read in the cdf file into a CDF object
    if(verbose)
      cat("Reading CDF file.\n")
    cdf <- read.cdffile(file.path(path.expand(cdf.path),filename), compress=compress)
    
    if(verbose)
      cat("Creating CDF environment\n")
  
    ## extract number or rows and number of columns
    sizex <- dim(cdf@name)[1]
    sizey <- dim(cdf@name)[2]
  
    ## where are the pm and mm
    pm.or.mm <- as.vector(pmormm(cdf))
    pmindex <- pm.or.mm
    mmindex <- !pm.or.mm
    rm(pm.or.mm)
    
    ## get ready to go
    ## start the new environment
    ## position in vector will correspond to the position on the array
    ## given ncol and nrow we can figure out the 2D (x,y) position
    ## now we assign 
    
    env = new.env(hash=TRUE, parent=emptyenv())
    
    genenames <- name.levels(cdf) ##so that we only use name.level method once
    n <- length(genenames) ##number of genes
    probenames <-  as.vector(cdf@name)
    position <- 1:length(probenames)
    numbers <- as.vector(cdf@atom)##number relates to position in genome
    tmp<- split(c(position,  pmindex,   mmindex,   numbers),
                c(probenames,probenames,probenames,probenames))
    rm(pmindex,mmindex,position,numbers,probenames)
    names(tmp) <- genenames[as.numeric(names(tmp))]
    
    ## this list will be converted to an environment using multiassign
    if(verbose)
      cat("Wait for about",round(length(tmp)/100),"dots")
    tmp <- lapply(tmp,function(x){
      if(verbose)
        if(runif(1)<0.01) cat(".")
      n <- length(x)/4
      position <- x[1:n] 
      pmindex <- which(x[(n+1):(2*n)]==1)
      mmindex <- which(x[(2*n+1):(3*n)]==1)
      numbers <- x[(3*n+1):(4*n)]
      
      ##get pm and mm and put in order according to numbers
      pm <- position[pmindex][order(numbers[pmindex])]
      mm <- position[mmindex][order(numbers[mmindex])]
      if(length(mm)==0)
        return(cbind(pm=pm,mm=rep(NA,length(pm))))
      else
        return(cbind(pm=pm,mm=mm))
    })
    
    if(verbose)
      cat("\n")
    
    multiassign(names(tmp),tmp, env)
    
    syms = list(
      I2XY  = paste("y*", sizex, "+x+1", sep=""),
      XY2I  = paste("r=cbind((i-1)%%",sizex,",(i-1)%/%",sizex,"); colnames(r)=c('x','y'); return(r)", sep=""),
      SIZEX = paste(sizex),
      SIZEY = paste(sizey),
      SIZEI = paste(sizex*sizey))
    
    ## return:
    if (return.env.only) {
      rv = env
    } else {
      rv = list(env=env, syms=syms)
    }
  } else {
    ## Binary CDF file("ReadCDFFile",
    tmp <- .Call("ReadCDFFile",file.path(path.expand(cdf.path),filename),PACKAGE="affyio")
    sizex <- tmp[[1]][2]  ## aka rows
    sizey <- tmp[[1]][1]  ##aka cols
    tmp <- tmp[[2]][order(names(tmp[[2]]))]

    env = new.env(hash=TRUE, parent=emptyenv())
    multiassign(names(tmp),tmp, env)
    syms = list(
      I2XY  = paste("y*", sizex, "+x+1", sep=""),
      XY2I  = paste("r=cbind((i-1)%%",sizex,",(i-1)%/%",sizex,"); colnames(r)=c('x','y'); return(r)", sep=""),
      SIZEX = paste(sizex),
      SIZEY = paste(sizey),
      SIZEI = paste(sizex*sizey))
    
    ## return:
    if (return.env.only) {
      rv = env
    } else {
      rv = list(env=env, syms=syms)
    }
    
  }



    
  return(rv)
}

##------------------------------------------------------------
## package maker
##------------------------------------------------------------
make.cdf.package<- function(filename,
                            packagename = NULL,
                            cdf.path = getwd(),
                            package.path = getwd(),
                            compress = FALSE,
                            author = "The Bioconductor Project",
                            maintainer = "Biocore Package Maintainer <maintainer@bioconductor.org>",
                            version = packageDescription("makecdfenv", fields="Version"),
                            species = NULL,
                            unlink  = FALSE,
                            verbose = TRUE) {
  
  ## If no packagename given change CDF filename to packagename.
  ## cleancdfname() is defined in package 'affy'.
  if(is.null(packagename))
    packagename <- cleancdfname(sub("\\.cdf$", "", filename, ignore.case=TRUE))

  if(is.null(species))
    stop("A species name must be specified, using the correct format:\n Example - Homo_sapiens\n")
  pkgroot <- sub("cdf$","", packagename)

  cdf <- make.cdf.env(filename, cdf.path=cdf.path, compress=compress, return.env.only=FALSE)
  assign(packagename, cdf$env)

  ## Add small env containing nrow and ncol
  dimenv <- new.env(hash = TRUE, parent = emptyenv())
  multiassign(c("NCOL", "NROW"), as.numeric(c(cdf$syms$SIZEX, cdf$syms$SIZEY)), dimenv)
  dimenvname <- paste(sub("cdf$", "", packagename), "dim", sep = "")
  assign(dimenvname, dimenv)

  home  <- path.package("makecdfenv")
  symbols = append(cdf$syms,
    list(PKGNAME     = packagename,
         VERSION     = version,
         AUTHOR      = author,
         TODAY       = date(),
         CDFFILENAME = filename,
         MAINTAINER  = maintainer,
         SPECIES     = species,
         PKGROOT     = pkgroot,
         DIMENVNAME  = dimenvname))

  ## see in the subdirectory 'Code' of this package for all the
  ## files that go into the package!
  res = createPackage(packagename,
                destinationDir = path.expand(package.path),
                originDir  = file.path(home, "Code"),
                symbolValues = symbols,
                unlink = unlink, quiet = !verbose)

  ## save an XDR file with the environment
  save(list = c(packagename, dimenvname),
       file = file.path(res$pkgdir, "data", paste(packagename,".rda",sep="")))

  pkgdir <- file.path(package.path, packagename)

  if(verbose)
      cat("\n\n\nREADME PLEASE:\n",
          "A source package has now been produced in\n", pkgdir, ".\n",
          "Before using this package it must be installed via 'R CMD INSTALL'\n",
          "at a terminal prompt (or DOS command shell).\nIf you are using ",
          "Windows, you will need to get set up to install packages.\n",
          "See the 'R Installation and Administration' manual, specifically\n",
          "Section 6 'Add-on Packages' as well as 'Appendix E: The Windows Toolset'\n",
          "for more information.\n\nAlternatively, you could use make.cdf.env(), ",
          "which will not require you to install a package.\n",
          "However, this environment will only persist for the current R session\n",
          "unless you save() it.\n\n\n",sep="")

}

xy2i = function(x,y) {
    .Deprecated("xy2indices",
                msg = paste("xy2i is deprecated and will be removed in the next BioC release.\n",
                "Use xy2indices in the affy package instead.", sep=""))
    @I2XY@
}
i2xy = function(i) {
    .Deprecated("indices2xy",
                msg = paste("i2xy is deprecated and will be removed in the next BioC release.\n",
                "Use indices2xy in the affy package instead.", sep=""))
    @XY2I@
}

# originally called 'getnature' in affyR

pmormm <- function(cdf) {
  a.i <- which(cdf@pbase.levels == "A")
  t.i <- which(cdf@pbase.levels == "T")
  g.i <- which(cdf@pbase.levels == "G")
  c.i <- which(cdf@pbase.levels == "C")
  md <- dim(cdf@name)

  # init to 'NA'
  nature <- matrix(NA,md[1],md[2])
  i <- which(
             ((cdf@pbase == a.i) & (cdf@tbase == t.i)) | 
             ((cdf@pbase == t.i) & (cdf@tbase == a.i)) |
             ((cdf@pbase == g.i) & (cdf@tbase == c.i)) |
             ((cdf@pbase == c.i) & (cdf@tbase == g.i))
             )
  # set the PM to TRUE
  nature[i] <- TRUE
  i <- which((! is.na(cdf@atom)) & is.na(nature))
  # set the MM to FALSE
  nature[i] <- FALSE
  return(nature)   
}

read.cdffile <- function(file, compress=FALSE) {
  ff <- new("Cdf")

  tmp <- getInfoInFile(file, "CDF", unit="Chip", property="Name", compress=compress)
  tmp <- substr(tmp, 1, nchar(tmp)-2)

  ff@cdfName <- tmp

  tmp <- .Call("readCDFfile", as.character(file), as.integer(3),
  as.integer(compress), PACKAGE="makecdfenv")
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)

  tmp <- factor(tmp)

  ff@name <- array(as.integer(tmp), mydim)

  ff@name.levels <- levels(tmp)
  rm(tmp)
  gc()
  tmp <- .Call("readCDFfile", as.character(file),
               as.integer(7), as.integer(compress), PACKAGE="makecdfenv")
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)

  tmp <- factor(tmp)

  ff@pbase <- array(as.integer(tmp), mydim)

  ff@pbase.levels <- levels(tmp)
  rm(tmp)
  gc()
  tmp <- .Call("readCDFfile", as.character(file),
               as.integer(8), as.integer(compress), PACKAGE="makecdfenv")
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)
  tmp <- factor(tmp)

  ff@tbase <- array(as.integer(tmp), mydim)

  ff@tbase.levels <- levels(tmp)
  rm(tmp)
  gc()

  tmp <- .Call("readCDFfile", as.character(file),
               as.integer(9), as.integer(compress), PACKAGE="makecdfenv")
  tmp[tmp == ""] <- NA
  mydim <- dim(tmp)

  ff@atom <- array(as.integer(tmp), mydim)

  gc()

  return(ff)
}

.onLoad <- function(libname, pkgname){
      where = asNamespace(pkgname)
      data(list = pkgname, package = pkgname, envir = where)
      packageStartupMessage(AnnotationDbi:::annoStartupMessages(pkgname))
}

## Laurent 2002 tneruaL
## Class definitions for 'Cdf'
setClass("Cdf", representation(cdfName="character",
                                 name="matrix",
                                 name.levels="character",
                                 pbase="matrix",
                                 pbase.levels="character",
                                 tbase="matrix",
                                 tbase.levels="character",
                                 atom="matrix"))
  ## name.levels method
    setGeneric("name.levels", function(object)
               standardGeneric("name.levels"))

   setMethod("name.levels","Cdf",function(object) object@name.levels)

  setGeneric("name.levels<-", function(object, value)
                 standardGeneric("name.levels<-"))
  setReplaceMethod("name.levels", "Cdf", function(object, value){
    object@name.levels <- value
    object
    })

  ## pbase method
    setGeneric("pbase", function(object)
               standardGeneric("pbase"))

   setMethod("pbase","Cdf",function(object) object@pbase)

  setGeneric("pbase<-", function(object, value)
                 standardGeneric("pbase<-"))
  setReplaceMethod("pbase", "Cdf", function(object, value){
    object@pbase <- value
    object
    })

 ##pbase.levels method
    setGeneric("pbase.levels", function(object)
               standardGeneric("pbase.levels"))

   setMethod("pbase.levels","Cdf",function(object) object@pbase.levels)

  setGeneric("pbase.levels<-", function(object, value)
                 standardGeneric("pbase.levels<-"))
  setReplaceMethod("pbase.levels", "Cdf", function(object, value){
    object@pbase.levels <- value
    object
    })

  ## tbase method
    setGeneric("tbase", function(object)
               standardGeneric("tbase"))

    setMethod("tbase","Cdf",function(object) object@tbase)

  setGeneric("tbase<-", function(object, value)
                 standardGeneric("tbase<-"))
  setReplaceMethod("tbase", "Cdf", function(object, value){
    object@tbase <- value
    object
    })

  ## tbase.levels method
    setGeneric("tbase.levels", function(object)
               standardGeneric("tbase.levels"))

   setMethod("tbase.levels","Cdf",function(object) object@tbase.levels)

  setGeneric("tbase.levels<-", function(object, value)
                 standardGeneric("tbase.levels<-"))
  setReplaceMethod("tbase.levels", "Cdf", function(object, value){
    object@tbase.levels <- value
    object
    })

  ## printing method for 'Cdf'
  setMethod("show", "Cdf",
            function(object) {
              ##print("Cdf object\n")
              cat("Cdf object:\n")
              cat("cdfName=", object@cdfName)
              cat("dim(name)=",dim(object@name)[1],"x",
                  dim(object@name)[2]," features\n",sep="")
              cat("nb. affyid=", length(object@name.levels), "\n", sep="")
            })

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{makecdfenv primer}
%\VignetteKeywords{Preprocessing, Affymetrix}
%\VignetteDepends{makecdfenv, affy}
%\VignettePackage{makecdfenv}
%documentclass[12pt, a4paper]{article}
\documentclass[12pt]{article}

\usepackage{amsmath}
\usepackage{hyperref}
\usepackage[authoryear,round]{natbib}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}
\author{Wolfgang Huber and Rafael Irizarry}
\begin{document}
\title{Textual description of makecdfenv}

\maketitle
\tableofcontents

\section{Introduction}
The \Rpackage{makecdfenv} package is part of the
Bioconductor\footnote{\url{http://www.bioconductor.org/}} project. It
can be used to create cdf environments from Affymetrix chip
description files to be used with the package \Rpackage{affy}.

For many {\it CDF} files a pre-assembled package is available from
\url{http://www.bioconductor.org/}. The
following notes explain what to do if you want to create your own.

\section{Creating a CDF package}
To install packages, you need certain tools besides R itself.  They
are generally available on most Unix platforms. Under MS-Windows, if
you use the precompiled binary distribution of R from CRAN, you will
need to install some extra tools, like the {\em source package
installation files} and {\em Perl}. If you think this is too
complicated, you can contribute the package to Bioconductor
and we will create a windows binary for you. Alternatively,
you may skip this section and proceed to
section~\ref{sec:make.cdf.env}

Let's say your CDF file is called \Robject{eggplantgenome.cdf} and
is in your working directory. To make a package, simply write
\begin{Sinput}
R> make.cdf.package("eggplantgenome.cdf", species = "Solanum_sp")
\end{Sinput}
This will create a subdirectory \Robject{eggplantgenomecdf} in your
working directory, which contains the package.  Please consult the
help page for \Rfunction{make.cdf.package} to find out about further
options.

Now, open a terminal with an operating system shell, and write
\begin{Sinput}
> R CMD INSTALL eggplantgenomecdf
\end{Sinput}
This will install the package into your R. You are now ready to use the
affy package to process eggplant genechips.

\section{Creating CDF environments}
\label{sec:make.cdf.env}
If you do not choose to use the package creation mechanism, you can
still produce a data structure (in R lingo, it is called an
{\it environment}) that can be used by the affy package.
Let's say your CDF file is called \Robject{EggPlantGenome.cdf} and
is in your working directory. Simply write
\begin{Sinput}
R> EggPlantGenome = make.cdf.env("EggPlantGenome.cdf")
\end{Sinput}
Please consult the help page for \Rfunction{make.cdf.env} to find out
about further options.

\section{Naming of CDF packages and environments}
What should you call your package or environment?

\Rfunction{make.cdf.package} chooses a default name by stripping all
non-letters and non-numbers from the CDF file name, and converting
everything to lower case. In almost all cases you should use this default
name for a package. 

This is important because the \Rpackage{affy} package contains
functionality to automatically detect the correct CDF package for a given
chip. If the package is not found, it will try to download from the 
Bioconductor servers. If the package is not found there, it will error out. 
At that point the only recourse is to manually change the cdfName slot of 
the \Robject{AffyBatch}, or re-create the CDF package with the correct name.

You can obtain the CDF name that is associated with a CEL file through
\begin{Sinput}
R> pname <- cleancdfname(whatcdf("mycelfile.cel"))
\end{Sinput}
Then call the package making function this way:
\begin{Sinput}
R> make.cdf.package("eggplantgenome.cdf", packagename=pname)
\end{Sinput}

Unfortunately, the naming convention for environments is slightly different.
Instead of using the \Rfunction{cleancdfname}, the \Rpackage{affy} package
will look for an environment with the same \it{exact} name as the original
CDF file, minus the suffix (the '.CDF' part). So for instance, if the CDF 
name was EggPlantGenome.CDF, you would want to create an environment this way:
\begin{Sinput}
R> EggPlantGenome <- make.cdf.env("EggPlantGenome.CDF")
\end{Sinput}

There are certain characters that will not work in a variable name
(such as +, -, /, *). If your CDF name contains one of these characters, the 
only recourse is to either create a package (which by default will not contain
these characters), or to manually change the cdfName of your \Robject{AffyBatch}
so the \Rpackage{affy} package will find your environment. Say your CDF name
is an-unacceptable-name.CDF.
\begin{Sinput}
R> mycdfenv <- make.cdf.env("an-unacceptable-name.CDF")
R> dat <- ReadAffy()
R> dat@cdfName <- "mycdfenv"
\end{Sinput}

In instances of \Robject{AffyBatch}, the \Robject{cdfName} slot gives the name
of the appropriate CDF file for the arrays that are represented in the
\Robject{intensity} slot.
The functions \Rfunction{read.celfile},
\Rfunction{read.affybatch}, and \Rfunction{ReadAffy} extract the CDF
filename from the CEL files that they read. The function
\Rfunction{cleancdfname} converts Affymetrix' CDF name to the name
that is used in Bioconductor. Here are two examples:
<<echo=F,results=hide>>=
library(affy)
@
<<>>=
cat("HG_U95Av2 is",cleancdfname("HG_U95Av2"),"\n")
cat("HG-133A is",cleancdfname("HG-133A"),"\n")
@
The method \Rfunction{getCdfInfo} takes as an argument \Robject{AffyBatch}
and returns the appropriate environment.  If \Robject{x} is an
\Robject{AffyBatch}, this function will look for an environment with name
\Robject{x@cdfName}.

\section{Location-ProbeSet Mapping}
\label{sec:probesloc}
On Affymetrix GeneChip arrays, several probes are used to represent
genes in the form of probe sets. From a {\it CEL} file we get for each
physical location, or cel, (uniquely identified by its $x$ and $y$
coordinates) an intensity.  The {\it CEL} file also contains the name
of the {\it CDF} file needed for the location-probe-set mapping. The
{\it CDF} files store the name of the probe set related to each
location on the array.  We store this mapping information in {\it R}
environments, such as the ones produced by \Rfunction{make.cdf.env} or
contained in the packages made by \Rfunction{make.cdf.package}.

In \Rpackage{affy}, the $x$ and $y$ coordinates are internally stored as
one number $i$. The mapping between $(x,y)$ and $i$ is provided by the
functions \Rfunction{i2xy(i)} and \Rfunction{xy2i} that are contained in each
CDF package. They are very simple:
\begin{eqnarray*}
  i &=& y  s_x + x + 1\\
  x &=& (i-1)\,\%\%\,  s_x\\
  y &=& (i-1)\,\%/\%\, s_x,
\end{eqnarray*}
where $s_x$ is the side length of the chip (measured in number of
probes) in $x$-direction.




\end{document}

\name{i2xy}
\alias{i2xy}
\alias{xy2i}
\title{Convert (x,y)-coordinates to single-number indices and back.}
\description{Convert (x,y)-coordinates on the chip (and in the CEL file) to the
single-number indices used in AffyBatch and CDF environment, and back.}
\usage{i2xy(i)
xy2i(x,y)
}
\arguments{
\item{x}{numeric. x-coordinate (from 1 to @SIZEX@)}
\item{y}{numeric. y-coordinate (from 1 to @SIZEY@)}
\item{i}{numeric. single-number index (from 1 to @SIZEI@)}
}
\details{Type \code{i2xy} and \code{xy2i} at the \R prompt to 
view the function definitions.}
\seealso{\code{\link{@PKGNAME@}}}
\keyword{datasets}

\examples{
  xy2i(5,5)
  i     = 1:(@SIZEX@*@SIZEY@)
  coord = i2xy(i)
  j     = xy2i(coord[, "x"], coord[, "y"])
  stopifnot(all(i==j))
  range(coord[, "x"])
  range(coord[, "y"])
}

\name{getInfoInFile}
\alias{getInfoInFile}
\title{get information from Affymetrix data files}
\description{
  Get specific information stored in Affymetrix data files (CEL or CDF).
}
\usage{
getInfoInFile(filename, type, unit, property, compress = NULL)
}
\arguments{
  \item{filename}{a file name }
  \item{type}{\code{"CEL"} or \code{"CDF"} are the only known types.}
  \item{unit}{the 'unit' to find the information}
  \item{property}{the 'property' of interest}
  \item{compress}{a boolean}
}
\details{
  The data files seem to have a structure of 'units'. A 'unit' start
  with something between square brackets (ex: "[UNIT1]"). The parameter
  \code{unit} let one specify in which unit the information of interest
  is located. The 'properties' are constituted of an identifier, the
  sign '=' and the value for the property (ex: "numCells=12000").

  If you are planning to use this function, I assume you know the what
  is in CEL and CDF files in details and that you know what are doing.
}
\value{
  a character with the value of the 'property'.
}
\note{
  The code goes through the file until the rigth 'unit' is found. Then
  it looks for the right 'property' further down (eventually going
  through the next units if the property is not found).
}
\seealso{\code{read.celfile}, \code{read.cdffile}, \code{whatcdf}}
\keyword{manip}

\name{make.cdf.env}
\alias{make.cdf.env}
\title{CDF Environment Maker}
\description{Reads an Affymetrix chip description file (CDF) and creates an
environment used as a hash table for the probe set mapping to location.}
\usage{
make.cdf.env(filename,
             cdf.path = getwd(),
             compress = FALSE,
             return.env.only = TRUE,
             verbose = TRUE)
}
\arguments{
  \item{filename}{Character. Filename of the CDF file -
    \bold{without} the path prefix!}
  \item{cdf.path}{Character. Path to the CDF file.}
  \item{compress}{Logical. If \code{TRUE}, CDF file is compressed.}
  \item{return.env.only}{Logical. If \code{TRUE} (the default),
    then the function returns an environment. Otherwise, a list with
    two elements, the first being the environment, and the second being
    a data structure that contains additional information needed for
    the package builder (see details, and vignette).}
  \item{verbose}{Logical. If \code{TRUE}, messages are shown.}
}

\value{Depending on the argument \code{return.env.only},
  either the environment, or a list with two elements,
  \code{env} and \code{syms}. See details.}

\details{Normally, this function should not be called directly. The
  preferred way to handle CDF information is to use
  \code{\link{make.cdf.package}} to build a package, and to
  install it into R. The CDF information can then either be invoked
  automatically by the package \code{affy}, or can be loaded manually by
  calling, for example, \code{library(hgu133a)}.
  
  Some R installations (typically on Windows) do not offer all the tools
  that are necessary for package building. In such situations, this
  function may be called directly. Please see the vignette for details
  - type:

  \code{openVignette("makecdfenv")}

  \bold{Return values:}  
  \code{env} is an environment, used as a hash table.
  For every probe set name we have a matrix with 2 columns. The first
  column contains the PM locations and the second column the MM
  locations. For PM only chips the MM column will have NAs.
  
  \code{syms} is a list that contains chip-specific (i.e., CDF-file
  specific) information that can be used in the construction of the help
  files for the CDF package.
}
\seealso{\code{\link{make.cdf.package}}}
\author{Rafael A. Irizarry, Wolfgang Huber}
\examples{
  env <- make.cdf.env("Hu6800.CDF.gz",
           cdf.path=system.file("extdata", package="makecdfenv"),
           compress=TRUE)
  length(ls(env))
  get("U53347_at", env)	   
}
\keyword{manip}

\name{make.cdf.package}
\alias{make.cdf.package}
\title{CDF Environment Package Maker}
\description{
This function reads an Affymetrix chip description file (CDF) and
creates an R package that when loaded has the CDF environment available
for use.}
\usage{
make.cdf.package(filename,
  packagename  = NULL,
  cdf.path     = getwd(),   
  package.path = getwd(),
  compress     = FALSE,
  author       = "The Bioconductor Project",
  maintainer   = "Biocore Package Maintainer <maintainer@bioconductor.org>",
  version      = packageDescription("makecdfenv", fields ="Version"),
  species      = NULL,
  unlink       = FALSE,
  verbose      = TRUE)
}
\arguments{
  \item{filename}{Character. Filename of the CDF file -
    \bold{without} the path prefix!}
  \item{packagename}{Character. Name wanted for the package.}
  \item{cdf.path}{Character. Path to the CDF file.}
  \item{package.path}{Character. Path where the package will be created.}
  \item{compress}{Logical. If \code{TRUE}, CDF file is compressed.}
  \item{author}{Character. What to put in the \code{author} field of the package.}
  \item{maintainer}{Character. What to put in the \code{maintainer} field of the package.}
  \item{version}{Character. What to put in the version
    field. Should be a of the form \code{x.x.x}.}
  \item{species}{Character. Must be specified using the format e.g., Homo\_sapiens}
  \item{unlink}{Logical. If \code{TRUE}, and a package directory exists
   already in \code{package.path}, that is overwritten.}
  \item{verbose}{Logical. If \code{TRUE} messages are shown.}
}
\details{The function is called for its side effect, creating a package.
By default the package name will be the name of the CDF file made lower
case and with special characters removed (i.e. only
alpha-numeric). 
  
In general one would want to use the name given in by
\code{cleancdfname(abatch@cdfName)} with \code{abatch} an
\code{\link[affy:AffyBatch-class]{AffyBatch}} object obtained, for
example, using \code{\link[affy:read.affybatch]{ReadAffy}}. This is the
package name that the 
\code{affy} package looks for by default.

If the user has a CEL file, called \code{filename}, the recommended
package name for the environment is
\code{cleancdfname(whatcdf(filename))}. This usually coincides with
the default.

Please see the vignette for more details.
}
\value{
If success, the function returns the name of the created package.
}
\seealso{\code{\link{make.cdf.env}}}
\author{Rafael A. Irizarry, Wolfgang Huber}
\examples{
  pkgpath <- tempdir()
  make.cdf.package("Hu6800.CDF.gz",
           cdf.path=system.file("extdata", package="makecdfenv"),
           compress=TRUE, species = "Homo_sapiens",
	   package.path = pkgpath)
  dir(pkgpath)	   
}

\keyword{manip}

\name{Cdf-class}
\docType{class}
\alias{Cdf-class}
\alias{atom}
\alias{atom<-}
\alias{name.levels}
\alias{name.levels<-}
\alias{pbase}
\alias{pbase<-}
\alias{pbase.levels}
\alias{pbase.levels<-}
\alias{tbase}
\alias{tbase<-}
\alias{tbase.levels}
\alias{tbase.levels<-}
\alias{atom,Cdf-method}
\alias{atom<-,Cdf-method}
\alias{name.levels,Cdf-method}
\alias{name.levels<-,Cdf-method}
\alias{pbase,Cdf-method}
\alias{pbase<-,Cdf-method}
\alias{pbase.levels,Cdf-method}
\alias{pbase.levels<-,Cdf-method}
\alias{tbase,Cdf-method}
\alias{tbase<-,Cdf-method}
\alias{tbase.levels,Cdf-method}
\alias{tbase.levels<-,Cdf-method}
\alias{show,Cdf-method}


\title{Class Cdf}
\description{A class describing the content a Chip Description File.}
\section{Creating Objects}{
  \code{  new('Cdf',}\cr
  \code{    cdfName      = ...., # Object of class character}\cr
  \code{    name         = ...., # Object of class matrix}\cr
  \code{    name.levels  = ...., # Object of class character}\cr
  \code{    pbase        = ...., # Object of class matrix}\cr
  \code{    pbase.levels = ...., # Object of class character}\cr
  \code{    tbase        = ...., # Object of class matrix}\cr
  \code{    tbase.levels = ...., # Object of class character}\cr
  \code{    atom         = ...., # Object of class matrix}\cr
  \code{  )}}
\section{Slots}{
  \describe{
    \item{\code{cdfName}:}{the CDF name tag. Used to link with the right
      CEL files}
    \item{\code{name}:}{Object of class "matrix" of 'factors' for the gene names corresponding to the probes.}
    \item{\code{name.levels}:}{Object of class "character" containing
      the levels corresponding to \code{name}.} 
    \item{\code{pbase}:}{Object of class "matrix" of pbase levels.}
    \item{\code{pbase.levels}:}{Object of class "character"  containing
      the levels corresponding to \code{pbase}. }
    \item{\code{tbase}:}{Object of class "matrix" of tbase levels.}
    \item{\code{tbase.levels}:}{Object of class "character" containing
      the levels corresponding to \code{tbase}.  }
    \item{\code{atom}:}{Object of class "matrix" of atom or probe numbers.}
  }
}

\details{
  The class \code{Cdf} was designed to store the content of Affymetrix's
  Chip Definition Files (CDF). This early class is no longer widely used
  in the package affy. Environments (called \code{cdfenvs}) are
  preferred to have an efficient mapping between probe set identifiers
  and indexes (needed to access/subset particular probe intensities).
  Unless one needs to access every information contained in a
  CDF file, the \code{cdfenvs} will be preferred.

  The following comments are only relevant to someone with interest in
  what is in a CDF file.
 The name associated
 to each probe is not unique, it corresponds to the gene name. It is
 very common to have a name repeated 40 times (20 perfect matches and 20
 mismatches). Probes can be grouped by pairs: a perfect
 match (PM) probe has its mismatch (MM) counterpart. The two probes in a
 pair differ by one base (usually located in the middle of the
 sequence). The information relative to this particular base are stored
 in \code{pbase} and \code{tbase}. At a given position (x,y), having
 pbase[x,y] == tbase[x,y] means having a MM while having pbase[x,y] !=
 tbase[x,y] means having a perfect mismatch if and only if one of the
 bases is A while the other is TRUE, or one is G and the other is C. The
 function 
 \code{\link{pmormm}} returns \code{TRUE} for PMs and \code{FALSE} for
 MMs.

 To know more about the file structure of a CDF file, one has to refer
 to the parsing code (mostly in C).
}
\seealso{
\code{\link{read.cdffile}},
\code{\link{make.cdf.env}},
\code{\link{make.cdf.package}}
}
\author{L. Gautier <laurent@cbs.dtu.dk>}

\section{Methods}{
  \describe{
    \item{atom}{(Cdf): An accesor function for the
      \code{ato,} slot. }
    \item{atom<-}{(Cdf):  A replacement function for
      \code{atom} slot. }
    \item{name.levels}{(Cdf):  An accesor function for the
      \code{name.levels} slot. }
    \item{name.levels<-}{(Cdf): A replacement function for
      \code{name.levels} slot. }
    \item{pbase}{(Cdf):  An accesor function for the
      \code{pbase} slot. }

    \item{pbase<-}{(Cdf): A replacement function for
      \code{pbase} slot. }
    \item{pbase.levels}{(Cdf):  An accesor function for the
      \code{pbase.levels} slot. }
    \item{pbase.levels<-}{(Cdf): A replacement function for
      \code{pbase.levels} slot. }
    \item{show}{(Cdf):  renders information about the \code{Cdf}
      object in a concise way on stdout. }
    \item{tbase}{(Cdf):  An accesor function for the
      \code{tbase} slot. }
    \item{tbase<-}{(Cdf): A replacement function for
      \code{tbase} slot. }
    \item{tbase.levels}{(Cdf):  An accesor function for the
      \code{tbase.levels} slot. }
    \item{tbase.levels<-}{(Cdf): A replacement function for
      \code{tbase.levels} slot. }
  }
}
\keyword{methods}

\name{pmormm}
\alias{pmormm}
\title{PM or MM}
\description{
  Determing  if intensities on an array from the cdffile are PM, MM, or neither.
}
\usage{
  pmormm(cdf)
}
\arguments{
  \item{cdf}{A \code{Cdf} object}
}
\details{
  The intensities on a chip fall in three categories: perfect match
  (PM), mismatch (MM) or unknown (No information about them is contained
  in the CDF file, yet they were found informative (details to come)).
  PM are coded as TRUE, MM as FALSE and unknown as NA.
}
\value{A matrix (see section 'details').}
\author{L. Gautier <laurent@cbs.dtu.dk>}
\keyword{manip}

\name{read.cdffile}
\alias{read.cdffile}
\title{Read a CDF file}
\description{Read the data contained in a CDF file}
\usage{read.cdffile(file, compress=FALSE)}
\arguments{
  \item{file}{the name of the CDF file}
  \item{compress}{whether the file is compressed or not}
}
\value{
  Returns a \code{\link{Cdf-class}} object.
}
\note{
  A \code{Cdf} object is not a cdf environment, which will be needed for
  the computation of expression values from the probe intensities in a
  \code{AffyBatch}. 
}
\details{This function is intended for use by
  \code{\link{make.cdf.env}}, which in turn is called by
  \code{\link{make.cdf.package}}. User may not have much
  benefit from calling this function directly.
  
  In order to save memory, the name corresponding to each value in the
  CEL is a factor. As in R \code{factor} objects
  cannot be also of type \code{matrix}, the names corresponding to the indices
  were stored in a vector of type \code{character} called
  \code{name.levels}.
  The same thing was done with the pbase and cbase information (more for
  consistency than by conviction that some memory
  could be saved here).
}
\author{Laurent Gautier (laurent@cbs.dtu.dk)}
\seealso{\code{\link{Cdf-class}}}
\keyword{file}
\examples{
fn <- system.file("extdata", "Hu6800.CDF.gz", package="makecdfenv")
mycdf <- read.cdffile(fn, compress=TRUE)
mycdf
}  
